import os, sys

from collections import namedtuple
from itertools import izip

from reads import fix_chrm_name_for_ucsc

from grit import config

FPKMTrackingLine = namedtuple( 
    'FPKMTrackingLine', 
    ["tracking_id", "class_code", "nearest_ref_id", 
     "gene_id", "gene_short_name", "tss_id", "locus", 
     "length"] )
    # "coverage", "FPKM", "FPKM_lo", "FPKM_hi", "status"

def build_gene_tracking_lines(gene):
    rv = []
    contig_name = gene.chrm
    if config.FIX_CHRM_NAMES_FOR_UCSC:
        contig_name = fix_chrm_name_for_ucsc(contig_name)
    
    for (mle, ub, lb, t) in izip( 
            mles, ubs, lbs, gene.transcripts):
        line = FPKMTrackingLine(
            t.id, # tracking ID
            '-' if t.class_code == None else t.class_code,  # class code
            '-' if t.ref_trans == None else t.ref_trans, # nearest ref id
            t.gene_id, # gene unique id
            '-' if t.ref_gene == None else t.ref_gene, # nearest reference gene
            '-', # TSS ID
            "%s:%s:%i-%i" % (contig_name, t.strand, t.start, t.stop), 
            str(t.calc_length()) # transcript length
            )
        rv.append(line)
    
    return rv
