def find_confidence_bounds_worker( gene, num_reads_in_bams,
                                   f_mat, mle_estimate, 
                                   trans_indices, trans_indices_lock,
                                   cb_alpha):
    bnd_type = 'LOWER' if bound_type == 'lb' else 'UPPER'

    if type(trans_index) == int:
        trans_indices = [trans_index,]
    else:
        assert isinstance( trans_index, list )
        trans_indices = trans_index
    
    # update the mle_estimate array to only store observable transcripts
    # add 1 to skip the out of gene bin
    observable_trans_indices = (
        numpy.array([-1,] + f_mat.transcript_indices().tolist())+1 )
    mle_estimate = mle_estimate[observable_trans_indices]
    
    config.log_statement( 
        "Estimating confidence bounds for gene %s" % gene.id )
    
    n_skipped = sum( 1 for x in sorted(f_mat.filtered_transcripts)
                     if x < trans_indices[0])
    # XXX Make sure that this is beign counted correctly
    #n_skipped_tmp = len(set(xrange(trans_indices[0])) - \
    #    set(x-1 for x in observable_trans_indices[1:] if x-1 < trans_indices[0]))
    #config.log_statement( str([n_skipped_tmp, n_skipped, f_mat.filtered_transcripts, \
    #    observable_trans_indices, trans_indices]), log=True)
    #assert n_skipped == n_skipped_tmp
    
    res = []
    while True:
        try:
            with trans_indices_lock:
                trans_index, exp_mat_row, cb_type = trans_indices.pop()
        except IndexError:
            config.log_statement("")
            return
        
        # make sure that this isn't a filtered transcript. if it is, return
        # the broadest transcript bounds possible.
        if trans_index+1 not in observable_trans_indices:
            if bound_type == 'lb': res.append((trans_index, 0.0))
            elif bound_type == 'ub': res.append((trans_index, 1.0))
            else: assert False
            n_skipped += 1
            continue

        if config.DEBUG_VERBOSE: config.log_statement( 
            "Estimating %s confidence bound for gene %s transcript %i/%i" % ( 
            bnd_type,gene.id,trans_index+1,len(gene.transcripts)))
        try:
            p_value, bnd = frequency_estimation.estimate_confidence_bound( 
                f_mat, num_reads_in_bams,
                trans_index-n_skipped+1, mle_estimate, bnd_type, cb_alpha )
        except Exception, inst:
            p_value = 1.
            bnd = 0.0 if bound_type == 'lb' else 1.0
            error_msg = "%i: Skipping %s (%s:%s:%i-%i): %s" % (
                os.getpid(), gene.id, 
                gene.chrm, gene.strand, gene.start, gene.stop, inst)
            config.log_statement( error_msg, log=True )
            if config.DEBUG_VERBOSE:
                config.log_statement( traceback.format_exc(), log=True )
        
        if config.DEBUG_VERBOSE: config.log_statement( 
            "FINISHED %s BOUND %s\t%s\t%i/%i\t%.2e\t%.2e" % (
            bnd_type, gene.id, None, 
            trans_index, len(gene.transcripts), 
            bnd, p_value ) )
        res.append((trans_index, bnd))
    
    config.log_statement( 
        "FINISHED Estimating %s confidence bound for gene %s transcript %i-%i/%i" % ( 
            bnd_type,gene.id,trans_indices[0]+1, trans_indices[-1]+1, 
            len(gene.transcripts)))
    
    return res

def worker( data, 
            promoter_reads,
            rnaseq_reads,
            polya_reads,
            fl_dists,
            write_design_matrices, 
            estimate_confidence_bounds):
    # reload all the reads, to make sure that this process has its
    # own file pointer
    for reads in ( promoter_reads, rnaseq_reads, polya_reads ):
        if reads != None: reads.reload()
    
    gene_id = None
    start_time = time.time()
    while True:
        # if we've been in this worker longer than an hour, then 
        # quit and re-spawn to free any un-reclaimed memory
        if time.time() - start_time > 3600:
            config.log_statement("")
            return
        # get the data to process
        try:
            work_type, gene_id, trans_indices = data.get_queue_item(gene_id)
        except Queue.Empty: 
            # if less than 5 minutes has passed, get a new gene
            if time.time() - start_time < 300:
                try: work_type, gene_id, trans_indices = data.get_queue_item()
                except Queue.Empty:
                    config.log_statement("")
                    return
            else:
                config.log_statement("")
                return
        
        if work_type == 'mle':
            gene = data.get_gene(gene_id)
            f_mat = data.get_design_matrix(gene.id)
            num_reads_in_bams = data.get_num_reads_in_bams()
            mle = estimate_mle_worker(gene, fl_dists, f_mat, num_reads_in_bams)
            if mle == None: 
                config.log_statement("")
                continue
            data.set_mle(gene, mle)
            if estimate_confidence_bounds:
                data.add_estimate_cbs_to_work_queue(gene.id)
            else:
                data.set_gene_to_finished(gene.id)
        else:
            assert work_type in ('ub', 'lb')

            gene = data.get_gene(gene_id)
            f_mat = data.get_design_matrix(gene.id)
            num_reads_in_bams = data.get_num_reads_in_bams()
            mle = data.get_mle(gene.id)
            
            cb_estimates = find_confidence_bounds_worker( 
                gene, fl_dists, num_reads_in_bams,
                f_mat, mle, 
                trans_indices, work_type, cb_alpha=config.CB_SIG_LEVEL)

            data.set_cbs(gene_id, work_type, cb_estimates)
        
        config.log_statement("")
    
    return

def write_gene_to_gtf( ofp, gene, mles=None, lbs=None, ubs=None, fpkms=None,
                       unobservable_transcripts=set()):
    #if mles != None:
        # we add one to accoutn for the out-of-gene transcript
    #    assert len(gene.transcripts) == len(mles)+len(unobservable_transcripts)
    n_skipped_ts = 0
    
    for index, transcript in enumerate(gene.transcripts):
        if index in unobservable_transcripts:
            n_skipped_ts += 1
            continue
        meta_data = {}
        if mles != None:
            meta_data["frac"] = ("%.2e" % mles[index-n_skipped_ts])
        if lbs != None:
            meta_data["conf_lo"] = "%.2e" % lbs[index-n_skipped_ts]
        if ubs != None:
            meta_data["conf_hi"] = "%.2e" % ubs[index-n_skipped_ts]
        if fpkms != None:
            meta_data["FPKM"] = "%.2e" % fpkms[index-n_skipped_ts]
        # choose the score to be the 1% confidence bound ratio, ie 
        # the current transcripts upper bound over all transcripts'
        # lower bound
        if lbs != None and ubs != None:
            frac = int((1000.*ubs[index-n_skipped_ts])/(1e-8+max(lbs)))
            transcript.score = max(1,min(1000,frac))
        
        if config.FIX_CHRM_NAMES_FOR_UCSC:
            transcript.chrm = fix_chrm_name_for_ucsc(transcript.chrm)
        
        ofp.write( transcript.build_gtf_lines(
                gene.id, meta_data, source="grit") + "\n" )
        ofp.flush()
    
    return

def write_gene_to_fpkm_tracking( ofp, gene, lbs=None, ubs=None, fpkms=None,
                       unobservable_transcripts=set()):
    n_skipped_ts = 0
    lines = []
    for index, transcript in enumerate(gene.transcripts):
        if index in unobservable_transcripts:
            n_skipped_ts += 1
            continue
        line = ['-']*13
        line[0] = str(transcript.id)
        line[3] = str(gene.id)
        line[6] = '%s:%i-%i' % ( gene.chrm, transcript.start, transcript.stop)
        line[7] = str( transcript.calc_length() )
        line[12] = 'OK'
        
        if fpkms != None:
            line[9] =  "%.2e" % fpkms[index-n_skipped_ts]
        if lbs != None:
            line[10] = "%.2e" % lbs[index-n_skipped_ts]
        if ubs != None:
            line[11] = "%.2e" % ubs[index-n_skipped_ts]
        
        lines.append( "\t".join(line) )
    
    ofp.write( "\n".join(lines)+"\n" )
    return

# output_dict, output_dict_lock, finished_genes_queue, 
def write_finished_data_to_disk( data, expression_ofp, fl_dists,
                                 compute_confidence_bounds, 
                                 write_design_matrices ):
    config.log_statement("Initializing background writer")
    while True:
        try:
            write_type, key = data.finished_queue.get(timeout=0.1)
            if write_type == 'FINISHED':
                break
        except Queue.Empty:
            config.log_statement( "Waiting for write queue to fill." )
            time.sleep( 1 )
            continue
        
        # write out the design matrix
        try:            
            config.log_statement( "Writing GENE %s to gtf" % key )

            gene = data.get_gene(key)
            unobservable_transcripts, lbs, mles, ubs, fpkms = (
                [], None, None, None, None)
            f_mat = data.get_design_matrix(key)
            unobservable_transcripts = sorted(
                f_mat.filtered_transcripts)
            
            mles = data.get_mle(key)[1:]
            lbs, ubs = None, None
            if compute_confidence_bounds:
                lbs = data.get_cbs(gene.id, 'lb')
                ubs = data.get_cbs(gene.id, 'ub')
            num_reads_in_bams = data.get_num_reads_in_bams(key)
            
            if ubs != None:
                ubs = calc_fpkm( gene, fl_dists, ubs,
                                 num_reads_in_bams)
            if lbs != None:
                lbs = calc_fpkm( gene, fl_dists, lbs,
                                 num_reads_in_bams)
            fpkms = calc_fpkm( 
                gene, fl_dists, mles, num_reads_in_bams)

            write_gene_to_fpkm_tracking( 
                expression_ofp, gene, lbs, ubs, fpkms, 
                unobservable_transcripts=unobservable_transcripts)

            # FIX THIS TO DELETE UNUSED DATA
            """
            with data.output_dict_lock:
                #del data.output_dict[(key, 'gene')]
                if not config.ONLY_BUILD_CANDIDATE_TRANSCRIPTS:
                    del data.output_dict[(key, 'mle')]
                    del data.output_dict[(key, 'design_matrices')]
                if compute_confidence_bounds:
                    del data.output_dict[(key, 'lbs')]
                    del data.output_dict[(key, 'ubs')]
            """
            config.log_statement( "" )
        except Exception, inst:
            config.log_statement( "Failed to write %s to GTF" % key, 
                                  log=True )
            config.log_statement( traceback.format_exc(), log=True )
        
    return

def convert_elements_to_arrays(all_elements):
    # convert into array
    all_array_elements = defaultdict( 
        lambda: defaultdict(lambda: numpy.zeros(0)) )
    for key, elements in all_elements.iteritems():
        for element_type, contig_elements in elements.iteritems():
            all_array_elements[key][element_type] \
                = numpy.array( sorted( contig_elements ) )

    return all_array_elements


def load_elements( fp ):
    all_elements = defaultdict( lambda: defaultdict(set) )
    for line in fp:
        if line.startswith( 'track' ): continue
        chrm, start, stop, element_type, score, strand = line.split()[:6]
        # subtract 1 from stop becausee beds are closed open, and we 
        # wnat everything in 0-based closed-closed
        all_elements[(chrm, strand)][element_type].add( 
            (int(start), int(stop)-1) )
    
    return convert_elements_to_arrays(all_elements)

def extract_elements_from_genes( genes ):
    all_elements = defaultdict( lambda: defaultdict(set) )
    for gene in genes:
        for key, val in gene.extract_elements().iteritems():
            all_elements[(gene.chrm, gene.strand)][key].update(val)

    
    return convert_elements_to_arrays( all_elements )
